﻿# ライブラリの定義
import numpy as np
import matplotlib.pyplot as plt

#初期値/パラメータ設定
A  = np.array([[0,0],[1,0.6]])
b  = np.array([[0],[0.64]])
c  = np.array([[0,1]])
Q  = 1   # システムノイズの分散
R  = 5   # 観測ノイズの分散
N  = 100
P  = np.eye(2)
xhat  = np.zeros([N,2])
xhatm = np.zeros([N,2])
G     = np.zeros([N,2])
x     = np.zeros([N,2])
y     = np.zeros([N,1])
y_t   = np.zeros([N,1])
np.random.seed(0)
u     = np.random.choice([-1,1],N)              # 入力
v     = np.random.normal(0,np.sqrt(Q),N)        # システムノイズ
w     = np.random.normal(0,np.sqrt(R),N)        # 観測ノイズ

#カルマンフィルタ動作
for k in range(1,N,1):
    x[k,:]      = (np.dot(A,x[k-1,:].reshape(2,1))+b*u[k-1] + b*v[k-1]).T 
    y[k]        = np.dot(c,x[k]) + 0.4*u[k] + w[k]     # 観測値
    y_t[k]      = np.dot(c,x[k]) + 0.4*u[k]            # 真値
    xhatm[k,:]  = (np.dot(A,xhat[k,:].reshape(2,1))+b*u[k]).T
    Pm          = np.dot(np.dot(A,P),A.T) + np.dot(np.dot(b,Q),b.T)
    G[k,:]      = (np.dot(Pm,c.T) / (np.dot(np.dot(c,Pm),c.T)+ R).reshape(-1)).T
    xhat[k,:]   = xhatm[k,:] + G[k,:]*(y[k] - np.dot(c,xhatm[k,:]))
    P           = np.dot((np.eye(2) - np.dot(G[k,:],c.T)),Pm)

#描画
fig = plt.figure()

plt.subplot(2,1,1)
plt.plot(np.arange(N) , u         , color = "black") # 入力

plt.subplot(2,1,2)
plt.plot(np.arange(N) , y_t       , color = "black") # 真値
plt.plot(np.arange(N) , y         , color = "red")   # 観測値
plt.plot(np.arange(N) , xhat[:,1] , color = "blue")  # 推定値

plt.show()

